# SWGui.py - ein einfaches GUI-Modul (Version 6)

# AENDERUNGEN gegenueber Version 5
# setCoords von SWCLine in SWCElement uebertragen
# startSWGui entfernt
# SWText erhaelt bei Aufruf mit undo=1 einen undo und einen redo Button
#   Vielleicht muesste man da nochmal drauf schauen wg. Layout (evtl.
#   uebergeortdeneter Frame?)

# -------------------------------------------

# Aenderungen gegenueber Version 4
# SWCElement setzt mycanvas schon in __init__ (spart aber keine Zeile)
# SWCElement hat move
# SWCElement hat stackUp, stackDown
# Bug in Rueckgabe von Checkbutton und Bittonlist beseitigt
# SWCLine: Koordinaten koennen als Liste oder einzelen angegeben werden
# Bei Farben koennen jetzt auch Listen angegeben werden, zB [255,0,0]

from Tkinter import *
import thread
import string

try:
    import Image
    import ImageTk
    thisSystemHasPIL=True
except Exception :
    thisSystemHasPIL=False
    
    

tkroot=None

def initSWGui(w=600,h=400):
    global tkroot
    tkroot=Tk()
    thread.start_new_thread(startSWGuiProc,())
    
def startSWGuiProc():
    global tkroot
    tkroot.update()
    tkroot.mainloop()

class SWFrameHori(Frame): 
    def __init__(self,container=tkroot):
        Frame.__init__(self,container)
        if isinstance(container, SWFrameHori):
            self.pack(side=LEFT)
        else: self.pack()

class SWFrameVert(Frame): 
    def __init__(self,container=tkroot):
        Frame.__init__(self,container)
        if isinstance(container, SWFrameHori):
            self.pack(side=LEFT)
        else: self.pack()

class SWButton(Button):
    def __init__(self, container=tkroot,text='', command=None, **options):
        Button.__init__(self,container,text=text,command=command)
        if isinstance(container, SWFrameHori):
            self.pack(side=LEFT)
        else: self.pack()
    def setText(self,s):
        self.configure(text=s)
    def setCommand(self,s):
        self.configure(command=s) 

class SWCheckbutton(Checkbutton): # select(), deselect() von  Button 
    def __init__(self, container=tkroot,text='', command=None, **options):
        self.var=IntVar()
        Checkbutton.__init__(self,container,text=text,command=command,variable=self.var)
        self.deselect()
        if isinstance(container, SWFrameHori):
            self.pack(side=LEFT)
        else: self.pack()
    def setText(self,s):
        self.configure(text=s) 
    def setCommand(self,s):
        self.configure(command=s)
    def getState(self): return self.var.get()==1

class SWRadiobuttonList():  
    def __init__(self, textList,container=tkroot, command=None):
        self.var=IntVar()
        self.buttons=[]
        for i in range(len(textList)):
            self.buttons.append(Radiobutton(container,variable=self.var,text=textList[i],value=i))
        if isinstance(container, SWFrameHori):
            for b in self.buttons: b.pack(side=LEFT)
        else:
            for b in self.buttons: b.pack()
    def setText(self,i,s):
        self.buttons[i].configure(text=s) 
    def setCommand(self,s):
        for b in self.buttons: b.configure(command=s)
    def getState(self): return self.var.get()


class SWLabel(Label):
    def __init__(self, container=tkroot,text='', **options):
        Label.__init__(self,container,text=text)
        if isinstance(container, SWFrameHori):
            self.pack(side=LEFT)
        else: self.pack()
    def setText(self,s):
        self.configure(text=s) 
    def setWidth(self,w):
        self.configure(width=w) 
    def setHeight(self,h):
        self.configure(height=h)
    def delete(self): self.mycanvas.delete(self.mytk)

class SWEntry(Entry): # get() liefert Text
    def __init__(self, container=tkroot,command=None,text='', **options):
        Entry.__init__(self,container,text=text)
        if isinstance(container, SWFrameHori):
            self.pack(side=LEFT)
        else: self.pack()
        self.setCommand(command)
    def setText(self,s):
        self.delete(0,END)
        self.insert(0,s)
    def getText(self):
        return Entry.get(self)
    def setCommand(self,cmd):
        if cmd!=None: self.bind("<Return>", cmd)

class SWText(Text): # width: Breite in Spalten, height: Hoehe in Zeilen
    def __init__(self,container=tkroot,height=5,width=20,undo=0):
        Text.__init__(self,container,height=5,width=20,undo=undo)
        if isinstance(container, SWFrameHori):
            self.pack(side=LEFT)
        else: self.pack()
        if undo==1:
            undoFrame = SWFrameHori()
            undoButton = SWButton(container=undoFrame, text="undo", command=self.edit_undo)
            redoButton = SWButton(container=undoFrame, text="redo", command=self.edit_redo)
    def getText(self):
        return Text.get(self,1.0,END)
    def addText(self,text,pos=END):
        Text.insert(self,pos,text)
    def setText(self,text):
        self.clear()
        self.addText(text)
    def clear(self):
        self.delete(1.0,END)

# ---------- Canvas ------------------------------------------------
        
class SWCanvas(Canvas):
    def __init__(self,container=tkroot,width=300,height=200):
        Canvas.__init__(self, container,width=width,height=height)
        if isinstance(container, SWFrameHori):
            self.pack(side=LEFT)
        else: self.pack()
    def createCircle(self,x,y,r): return SWCCircle(self,x,y,r)
    def createLine(self,x1,y1,x2,y2): return SWCLine(self,x1,y1,x2,y2)
    def createText(self,x,y,t): return SWCText(self,x,y,t)
    def createRectangle(self,x1,y1,x2,y2): return SWCRectangle(self,x1,y1,x2,y2)
    def createPolygon(self,*punkte): return SWCPolygon(self,punkte)
    def createArc(self,x1,y1,x2,y2): return SWCArc(self,x1,y1,x2,y2)
    def createOval(self,x1,y1,x2,y2): return SWCOval(self,x1,y1,x2,y2)    
    def createImage(self,x,y,img): return SWCImage(self,x,y,img)
    def getDim(self):
        return [int(self.cget("width")),int(self.cget("height"))]
    def setClickCommand(self,cmd):
        self.bind("<Button-1>",cmd)
    def setReleaseCommand(self,cmd):
        self.bind("<ButtonRelease-1>",cmd)
    def setDragCommand(self,cmd):
        self.bind("<B1-Motion>",cmd)

# ---------- Canvas Elemente ----------------------------------------

class SWCElement():
    def __init__(self,mycanvas):
        self.mycanvas=mycanvas
        self.mytk=None
    def setBorderColor(self,col):
        self.mycanvas.itemconfigure(self.mytk,outline=mkColor(col))
    def setColor(self,col):
        self.mycanvas.itemconfigure(self.mytk,fill=mkColor(col))
    def delete(self): self.mycanvas.delete(self.mytk)
    def stackUp(self):
        self.mycanvas.tag_raise(self.mytk)
    def stackDown(self):
        self.mycanvas.tag_lower(self.mytk)
    def setCoords(self,*rest):
        rest=list(rest)
        if len(rest)==1: coords=rest[0]
        else:
            if len(rest)==4: coords=rest
            else:
                raise Exception("SWCElement.setCoords: wrong arguments")
        self.coords=coords
        self.mycanvas.coords(self.mytk,coords[0],coords[1],coords[2],coords[3])
    def move(self,*args):
        args=list(args)
        if len(args)==2: [dx,dy]=args
        else:
            if len(args)==1 and isinstance(args[0],list):
                [dx,dy]=args[0]
            else: raise Exception("SWCElement.move: wrong argument types")
        self.mycanvas.move(self.mytk,dx,dy)

def mkColor(*args):
    if len(args)==1:
        if isinstance(args[0],str): return args[0]
        if isinstance(args[0],list):
            if len(args[0])==3:
                return string.replace("#%2x%2x%2x"%(args[0][0],args[0][1],args[0][2])," ","0")
    if len(args)==3:
        print "3-fall", args
        return string.replace("#%2x%2x%2x"%(args[0],args[1],args[2])," ","0")
    raise Exception("Color misspecified")
    

class SWCCircle(SWCElement):
    def __init__(self,canvas,x,y,r):
        SWCElement.__init__(self,canvas)
        self.mytk=canvas.create_oval(x-r,y-r,x+r,y+r)
        self.radius=r
        self.x=x
        self.y=y
    def setRadius(self,r):
        self.radius=r
        self.mycanvas.coords(self.mytk,self.x-self.radius,self.y-self.radius,self.x+self.radius,self.y+self.radius)
    def setCenter(self,x,y):
        self.x=x
        self.y=y
        self.mycanvas.coords(self.mytk,int(x-self.radius),int(y-self.radius),int(x+self.radius),int(y+self.radius))
    def setCoords(self,*rest):
        rest=list(rest)
        if len(rest)==1: coords=rest[0]
        else:
            if len(rest)==4: coords=rest
            else:
                raise Exception("SWCCircle.setCoords: wrong arguments")
        self.coords=coords
        if coords[2]-coords[0]==coords[3]-coords[1]:
            self.mycanvas.coords(self.mytk,coords[0],coords[1],coords[2],coords[3])
        else: raise Exception("SWCCircle.setCoords: No Circle")

class SWCLine(SWCElement):
    def __init__(self,canvas,x1,y1,x2,y2):
        SWCElement.__init__(self,canvas)
        self.mytk=canvas.create_line(x1,y1,x2,y2)
        self.coords=[x1,y1,x2,y2]
    def setWidth(self,w): self.mycanvas.itemconfigure(self.mytk,width=w)

class SWCRectangle(SWCElement):
    def __init__(self,canvas,x1,y1,x2,y2):
        SWCElement.__init__(self,canvas)
        self.mytk=canvas.create_rectangle(x1,y1,x2,y2)
        self.coords=[x1,y1,x2,y2]
    def setWidth(self,w): self.mycanvas.itemconfigure(self.mytk,width=w)

class SWCPolygon(SWCElement):
   def __init__(self,canvas,*punkte):
      SWCElement.__init__(self,canvas)
      self.mytk=canvas.create_polygon(punkte,fill="",outline="black")
      self.coords=punkte
   def setCoords(self,*rest):   # Hier stimmt noch was nicht
      liste=list(rest)
      if len(liste)==1:
          coords=liste[0]
          apply(self.mycanvas.coords, [self.mytk]+coords)
      else:
        if len(liste)%2==0: coords=liste
        else:
            raise Exception("SWCPolygon: wrong arguments")
        self.coords=coords
        apply(self.mycanvas.coords, [self.mytk]+coords)
   def setWidth(self,w): self.mycanvas.itemconfigure(self.mytk,width=w)

class SWCArc(SWCElement):
    def __init__(self, canvas,x1,y1,x2,y2, start=45, extent=90):
        SWCElement.__init__(self,canvas)
        self.coords=[x1,y1,x2,y2]
        self.mytk=canvas.create_arc(self.coords[0],self.coords[1],self.coords[2],self.coords[3], start=45, extent=90)
    def setExtent(self,ex): self.mycanvas.itemconfigure(self.mytk,extent=ex)
    def setStart(self, st): self.mycanvas.itemconfigure(self.mytk,start=st)
    def setStyle(self, sty): self.mycanvas.itemconfigure(self.mytk,style=sty)

class SWCOval(SWCElement):
    def __init__(self,canvas,x1,y1,x2,y2):
        SWCElement.__init__(self,canvas)
        self.mytk=canvas.create_oval(x1,y1,x2,y2)
        self.coords=[x1,y1,x2,y2]
        self.width=abs(x2-x1)
        self.height=abs(y2-y1)
        self.center=[(x2-x1)/2,(y2-y1)/2]
    def setCenter(self,x,y):
        self.coords[0]=int(x-self.width/2)
        self.coords[1]=int(y-self.height/2)
        self.coords[2]=int(x+self.width/2)
        self.coords[3]=int(y+self.height/2)
        self.mycanvas.coords(self.mytk,self.coords[0],self.coords[1],self.coords[2],self.coords[3])
    def setFillColor(self,col): self.mycanvas.itemconfigure(self.mytk,fill=col)


class SWCText(SWCElement):
    def __init__(self,canvas,x,y,text):
        SWCElement.__init__(self,canvas)
        self.mytk=canvas.create_text(x,y,text=text)
        self.coords=[x,y]
    def setCoords(self,coords):
        self.coords=coords
        self.mycanvas.coords(self.mytk,coords[0],coords[1])
    def setText(self,text): self.mycanvas.itemconfigure(self.mytk,text=text)

class SWCImage(SWCElement):
    def __init__(self,canvas,x,y,img):
        global thisSystemHasPIL
        SWCElement.__init__(self,canvas)
        if isinstance(img,str): # Zeichenkette, muss Dateiname sein
            if thisSystemHasPIL:
                self.myPILimgOriginal=Image.open(img).convert("RGBA")
                self.myPILimg=Image.open(img).convert("RGBA")
                img=ImageTk.PhotoImage(self.myPILimg)
            else:
                self.myPILimgOriginal=None
                self.myPILimg=None
                img = PhotoImage(file=img)
        self.myimg=img
        self.mytk=canvas.create_image(x,y,image=img)
        self.coords=[x,y]
    def setCoords(self,coords):
        self.coords=coords
        self.mycanvas.coords(self.mytk,coords[0],coords[1])
    def getDim(self):
        return [self.myimg.width(),self.myimg.height()]
    def rotate(self,angle):
        global thisSystemHasPIL
        if not(thisSystemHasPIL): raise Exception("No PIL")
        self.myPILimg=self.myPILimg.rotate(angle)
        self.myimg=ImageTk.PhotoImage(self.myPILimg)
        self.mycanvas.delete(self.mytk)
        self.mytk=self.mycanvas.create_image(self.coords[0],self.coords[1],
                                      image=self.myimg)
        
# ----- Scale ------------
    
class SWScale(Scale):
    def __init__(self,ori,container=tkroot,minimum=0,maximum=100):
        # ori ist "vertical" oder "horizontal"
        Scale.__init__(self, container)
        self.mycontainer=container
        self.mini=minimum
        self.maxi=maximum
        self.configure(from_=minimum,to=maximum)
        self.configure(orient=ori)
        if isinstance(container, SWFrameHori):
            self.pack(side=LEFT)
        else: self.pack()
    def setCommand(self,cmd):
        self.configure(command=cmd)
        

class SWScaleHori(SWScale):
    def __init__(self,container=tkroot,minimum=0,maximum=100):
        SWScale.__init__(self,"horizontal",container,minimum,maximum)

class SWScaleVert(SWScale):
    def __init__(self,container=tkroot,minimum=0,maximum=100):
        SWScale.__init__(self,"horizontal",container,minimum,maximum)


# --- Dialoge --------------

def SWListChoiceDialog(liste): # Waehle ein Element aus einer Liste aus
    # im wesentlichen aus Python Cookbook 
    return ListBoxChoice(tkroot, list=liste).returnValue()

class ListBoxChoice(object):
    def __init__(self, master=None, title=None, message=None, list=[]):
        self.master = master
        self.value = None
        self.list = list[:]
        
        self.modalPane = Toplevel(self.master)

        self.modalPane.transient(self.master)
        self.modalPane.grab_set()

        self.modalPane.bind("<Return>", self._choose)
        self.modalPane.bind("<Escape>", self._cancel)

        if title:
            self.modalPane.title(title)

        if message:
            Label(self.modalPane, text=message).pack(padx=5, pady=5)

        listFrame = Frame(self.modalPane)
        listFrame.pack(side=TOP, padx=5, pady=5)
        
        scrollBar = Scrollbar(listFrame)
        scrollBar.pack(side=RIGHT, fill=Y)
        self.listBox = Listbox(listFrame, selectmode=SINGLE)
        self.listBox.pack(side=LEFT, fill=Y)
        scrollBar.config(command=self.listBox.yview)
        self.listBox.config(yscrollcommand=scrollBar.set)
        self.list.sort()
        for item in self.list:
            self.listBox.insert(END, item)

        buttonFrame = Frame(self.modalPane)
        buttonFrame.pack(side=BOTTOM)

        chooseButton = Button(buttonFrame, text="Choose", command=self._choose)
        chooseButton.pack()

        cancelButton = Button(buttonFrame, text="Cancel", command=self._cancel)
        cancelButton.pack(side=RIGHT)

    def _choose(self, event=None):
        try:
            firstIndex = self.listBox.curselection()[0]
            self.value = self.list[int(firstIndex)]
        except IndexError:
            self.value = None
        self.modalPane.destroy()

    def _cancel(self, event=None):
        self.modalPane.destroy()
        
    def returnValue(self):
        self.master.wait_window(self.modalPane)
        return self.value


